#!/usr/bin/perl -T

use strict;
use warnings;

#*******************************************************************************
# Data::Filter::Formater module/class tests
#
# Author:   Jan Mach, jan.mach@cesnet.cz
# Version:  0.1
# Purpose:
#
#*******************************************************************************

use Data::Dumper;
#use Smart::Comments;

use Test::More tests => 16;

# First test if the module correctly compiles
BEGIN {
        use vars qw($TESTED_CLASS);
        $TESTED_CLASS = 'Data::Filter::Formater';

        use_ok('Tester');
        use_ok('Data::Filter::Traverser');
        use_ok('Data::Filter::Factory');
        use_ok($TESTED_CLASS);
    }

# Debug output explicit ON switch
#Tester::DEBUG_ON();
#Tester::DEBUG_ON(10);

# Debug output explicit OFF switch
#Tester::DEBUG_OFF();

# User should know, which version is beeing tested (just for sure)
diag("$TESTED_CLASS version: " . $Data::Filter::Formater::VERSION) if Tester::D();

# Now attempt to create a class instance and check it
my $instance = $TESTED_CLASS->new();
isa_ok($instance, $TESTED_CLASS);

# View the created instance
diag("Instance dump 01: " . Dumper($instance)) if Tester::D(2);

# Instantinate the filter factory object
my $ffactory = Data::Filter::Factory->new_from_file('./conf/filter-grammar.conf');
isa_ok($ffactory, 'Data::Filter::Factory');

#-------------------------------------------------------------------------------
# Create filters by using rules and compile them afterwards
#-------------------------------------------------------------------------------

my @RULES = (
        [
            'Source/IP4 == "195.113.144.194"',
            '(Source/IP4 EQ "195.113.144.194")',
            '<div class="filter-tree"><div class="filter-operation"><h3>EQ</h3><ul class="optree"><li><div class="filter-variable">Source/IP4</div></li><li><div class="filter-constant">195.113.144.194</div></li></ul></div></div>',
        ],
        [
            'CreateTime <= "2012-10-09T10:12:06Z"',
            '(CreateTime LE "2012-10-09T10:12:06Z")',
            '<div class="filter-tree"><div class="filter-operation"><h3>LE</h3><ul class="optree"><li><div class="filter-variable">CreateTime</div></li><li><div class="filter-constant">2012-10-09T10:12:06Z</div></li></ul></div></div>',
        ],
        [
            '(2 < 5) AND (4 > 2)',
            '(("2" LT "5") AND ("4" GT "2"))',
            '<div class="filter-tree"><div class="filter-operation"><h3>AND</h3><ul class="optree"><li><div class="filter-operation"><h3>LT</h3><ul class="optree"><li><div class="filter-constant">2</div></li><li><div class="filter-constant">5</div></li></ul></div></li><li><div class="filter-operation"><h3>GT</h3><ul class="optree"><li><div class="filter-constant">4</div></li><li><div class="filter-constant">2</div></li></ul></div></li></ul></div></div>',
        ],
        [
            '(2 + 10) + ((10 - 2) * 3)',
            '(("2" ADD "10") ADD (("10" SUB "2") MUL "3"))',
            '<div class="filter-tree"><div class="filter-operation"><h3>ADD</h3><ul class="optree"><li><div class="filter-operation"><h3>ADD</h3><ul class="optree"><li><div class="filter-constant">2</div></li><li><div class="filter-constant">10</div></li></ul></div></li><li><div class="filter-operation"><h3>MUL</h3><ul class="optree"><li><div class="filter-operation"><h3>SUB</h3><ul class="optree"><li><div class="filter-constant">10</div></li><li><div class="filter-constant">2</div></li></ul></div></li><li><div class="filter-constant">3</div></li></ul></div></li></ul></div></div>',
        ],
        [
            '(ID like ("abc" . "123"))',
            '(ID LIKE ("abc" CON "123"))',
            '<div class="filter-tree"><div class="filter-operation"><h3>LIKE</h3><ul class="optree"><li><div class="filter-variable">ID</div></li><li><div class="filter-operation"><h3>CON</h3><ul class="optree"><li><div class="filter-constant">abc</div></li><li><div class="filter-constant">123</div></li></ul></div></li></ul></div></div>',
        ],
    );

my $counter = 0;
my ($filter, $result);
foreach my $rule (@RULES)
{
    ++$counter;
    diag("Initiating filter test A$counter\n") if Tester::D();
    $filter = $ffactory->filter_from_string($rule->[0]);
    is($filter->to_string(), $rule->[1], "Filter test A$counter: $rule->[0]");
    diag("FILTER A$counter: " . Dumper($filter) . "\n") if Tester::D();
    $result = $instance->format_html($filter);
    is($result, $rule->[2], "Filter test B$counter: $rule->[0]");
    diag("FILTER B$counter: " . Dumper($result) . "\n") if Tester::D();
}
