#!/usr/bin/perl -T

use strict;
use warnings;

#*******************************************************************************
# Data::Query::Mongo class tests
#
# Author:   Jan Mach, jan.mach@cesnet.cz
# Version:  0.1
# Purpose:
# Comments:
#*******************************************************************************

use Data::Dumper;
#use Smart::Comments;

use Test::More tests => 17;

# First test if the module correctly compiles
BEGIN {
        use vars qw($TESTED_CLASS);
        $TESTED_CLASS = 'Data::Query::Mongo';

        use_ok('Tester');
        use_ok('Data::Filter::Traverser');
        use_ok('Data::Query');
        use_ok('Data::Filter::Factory');
        use_ok('Data::Tree');
        use_ok('Data::Convertor::Mongo');
        use_ok('Mentat::Storage::Mongo');
        use_ok('Data::Path::Value');
        use_ok($TESTED_CLASS);
    }

# Debug output explicit ON switch
#Tester::DEBUG_ON();
#Tester::DEBUG_ON(10);

# Debug output explicit OFF switch
#Tester::DEBUG_OFF();

# User should know, which version is beeing tested (just for sure)
diag("$TESTED_CLASS version: " . $Data::Query::Mongo::VERSION) if Tester::D();

# Now attempt to create a class instance and check it
my $instance = $TESTED_CLASS->new(convertor => Data::Convertor::Mongo->new(),
                                  storage => Mentat::Storage::Mongo->new(database => 'unittest', collection => 'query'));
isa_ok($instance, $TESTED_CLASS);

# View the created instance
diag("Instance dump 01: " . Dumper($instance)) if Tester::D();

# Filter factory instance
my $ffactory = Data::Filter::Factory->new_from_file('../../conf/filter-grammar.conf');
isa_ok($ffactory, 'Data::Filter::Factory');

# View the created instance
diag("Instance dump 02: " . Dumper($ffactory)) if Tester::D();

#-------------------------------------------------------------------------------

my @FILTERS = (
        [
            'ID == "1-1340627016.850910-omnius.cesnet.cz-pickup-20B2"',
            "{'ID' => '1-1340627016.850910-omnius.cesnet.cz-pickup-20B2'};",
        ],
        [
            'EXISTS ID',
            "{'ID' => {'\$exists' => 1}};",
        ],
        [
            'ID LIKE "abc"',
            "{'ID' => qr/(?^:abc)/};",
        ],
        [
            'Source/IP4 eq "222.236.44.28"',
            "{'Source.IP4' => '222.236.44.28'};",
        ],
        [
            '(Source/IP4 eq "222.236.44.28") and (ts_u < "2013-05-30 10:21:09")  and (ts_u > "2013-05-23 10:21:09")',
            "{'\$and' => [{'Source.IP4' => '222.236.44.28'},{'\$and' => [{'ts_u' => {'\$lt' => '2013-05-30 10:21:09'}},{'ts_u' => {'\$gt' => '2013-05-23 10:21:09'}}]}]};",
        ],
        [
            'CreateTime < "2013-05-30 10:21:09"',
            "{'CreateTime' => {'\$lt' => '2013-05-30 10:21:09'}};",
        ],
    );

# Create and test couple of filters
my ($filter, $query, $query_s);
my $counter = 0;
foreach my $rule (@FILTERS)
{
    ++$counter;
    $filter = $ffactory->filter_from_string($rule->[0]);
    diag("Filter $counter: " . Dumper($filter)) if Tester::D();
    $query = $instance->generate_query($filter);
    diag("Query $counter: " . Dumper($query)) if Tester::D();
    $query_s = Tester::P_C($query);
    $query_s =~ s/'data' => '[^}]+\}/'data' => '???'}/g;
    $query_s =~ s/'subtype' => 0,?//g;
    is($query_s, $rule->[1], "MongoDB query test $counter");
    diag("Result $counter: " . Dumper($instance->query($filter))) if Tester::D();
}

eval {
    $filter = $ffactory->filter_from_string('absdefg');
    diag("Filter 05: " . Dumper($filter)) if Tester::D();
    diag("Query 05: " . Dumper($instance->generate_query($filter))) if Tester::D();
    diag("Result 05: " . Dumper($instance->query($filter))) if Tester::D();
};
if ($@)
{
    diag("ERROR: $@\n");
}
